
angular.module('voicePersonals', ['ionic', 'voicePersonals.controllers', 'voicePersonals.services', 'voicePersonals.directives'])

.run(function($ionicPlatform, vpAgeVerification, vpSplashScreen, $rootScope, vpConfig) {
    $ionicPlatform.ready(function() {

        // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
        // for form inputs)
        if(window.cordova && window.Keyboard) {
          window.Keyboard.hideKeyboardAccessoryBar(true);
        }
        if(window.StatusBar) {
          // org.apache.cordova.statusbar required
          StatusBar.styleDefault();
        }

        //add screen locking and age verification
        if (window.cordova) {
            var cordovaReady = function() {
                vpSplashScreen.hideSplashScreen(false);
                document.addEventListener("resume", vpAgeVerification.verifyAge, false);
                document.addEventListener("pause", vpAgeVerification.pauseCordova, false);
                vpAgeVerification.verifyAge();
                $rootScope.$on('$stateChangeSuccess', function(event, toState, toParams, fromState, fromParams) {  //some age verification state loaded
                    vpSplashScreen.startHideScreenCheck();
                });
            }
            document.addEventListener("deviceready", cordovaReady, false);
        }  //developing locally with ionic serve, cordova device ready event does not exist
        else vpAgeVerification.verifyAge();

    });
})

.config(function($stateProvider, $urlRouterProvider, $httpProvider) {

    // Ionic uses AngularUI Router which uses the concept of states
    // Learn more here: https://github.com/angular-ui/ui-router
    // Set up the various states which the app can be in.
    // Each state's controller can be found in controllers.js
    $stateProvider


    //setup states for 2257 age verification and screen lock
    .state('verifyage', {
        url: "/verifyage",
        templateUrl:  "templates/verify-age.html",
        controller: "VerifyAgeCtrl"
    })

    .state('setupPINSecurity', {
        url: "/setuppinsecurity",
        templateUrl:  "templates/setup-pin-security.html",
        controller: "setupPINSecurityCtrl"
    })

    .state('verifypin', {
        url: "/verifypin",
        templateUrl:  "templates/verify-pin.html",
        controller: "VerifyPINCtrl"
    })


    // setup an abstract state for the tabs directive and add application wide controller functionality
    .state('tab', {
        url: "/tab",
        abstract: true,
        templateUrl:  "templates/tabs.html",
        controller: "vpApplicationCtrl"
    })

    // Each tab has its own nav history stack:

    .state('tab.browse', {
      abstract: true,
      views: {
        'tab-browse': {
          templateUrl: 'templates/tab-browse.html',
          controller: 'BrowsePersonalsCtrl'
        }
      }
    })

    .state('tab.browse.personals', {
      url: '/browse',
      views: {
          '': {
              templateUrl: 'templates/tab-browse-personals.html'
          }
      }
    })

    .state('tab.inbox', {
      url: '/inbox?refresh',
      views: {
        'tab-friends': {
          templateUrl: 'templates/tab-inbox.html',
          controller: 'InboxCtrl'
        }
      }
    })

    .state('tab.inbox-thread', {
      url: '/inbox/:messageID',
      views: {
        'tab-friends': {
          templateUrl: 'templates/inbox-thread.html',
          controller: 'InboxThreadCtrl'
        }
      }
    })

    .state('tab.profile-enter-redirect', { // view personal or login/create personal
        url: '/profile-enter-redirect',
        views: {
            'tab-account': {
                templateUrl: 'templates/blank.html',
                controller: 'ProfileEnterRedirectCtrl'
            }
        }
    })

    .state('tab.profile', {
      url: '/profile',
      views: {
        'tab-account': {
          templateUrl: 'templates/tab-profile.html',
          controller: 'ProfileCtrl'
        }
      }
    })

    .state('tab.profile-edit', {
        url: '/profile-edit',
        views: {
            'tab-account': {
                templateUrl: 'templates/tab-profile-edit.html',
                controller: 'ProfileEditCtrl'
            }
        }
    })

    .state('tab.login', {
      url: '/login',
      views: {
          'tab-account': {
              templateUrl: 'templates/login.html',
              controller: 'LoginCtrl'
          }
      }
    })

    .state('tab.create-account', {
      url: '/create-account',
      views: {
          'tab-account': {
              templateUrl: 'templates/create-account.html',
              controller: 'CreateAccountCtrl'
          }
      }
    });

    // if none of the above states are matched, use this as the fallback
    //$urlRouterProvider.otherwise('/verifyage');


    $httpProvider.defaults.cache = false;

    //implement hooks into $httpProvider to convert default JSON data to URL encoded POST data expected by PHP
    $httpProvider.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded;charset=utf-8';
    /**
     * The workhorse; converts an object to x-www-form-urlencoded serialization.
     * @param {Object} obj
     * @return {String}
     */
    var param = function ( obj ) {
        var query = '',
            name, value, fullSubName, subName, subValue, innerObj, i;

        for ( name in obj ) {
            value = obj[ name ];

            if ( value instanceof Array ) {
                for ( i = 0; i < value.length; ++i ) {
                    subValue = value[ i ];
                    fullSubName = name + '[' + i + ']';
                    innerObj = {};
                    innerObj[ fullSubName ] = subValue;
                    query += param( innerObj ) + '&';
                }
            } else if ( value instanceof Object ) {
                if (value instanceof Date) {
                    query += encodeURIComponent(name) + '=' + encodeURIComponent(value.toISOString()) + '&';
                } else {
                    for ( subName in value ) {
                        subValue = value[ subName ];
                        fullSubName = name + '[' + subName + ']';
                        innerObj = {};
                        innerObj[ fullSubName ] = subValue;
                        query += param( innerObj ) + '&';
                    }
                }
            } else if ( value !== undefined && value !== null ) {
                query += encodeURIComponent( name ) + '=' + encodeURIComponent( value ) + '&';
            }
        }
        return query.length ? query.substr( 0, query.length - 1 ) : query;
    };

    // Override $http service's default transformRequest
    $httpProvider.defaults.transformRequest = [
        function ( data ) {
            return angular.isObject( data ) && String( data ) !== '[object File]' ? param( data ) : data;
        }
    ];

});

