angular.module('voicePersonals.controllers', [])

.controller('LoginCtrl', function($scope, vpSystemMessage, vpAccount, $state, vpSession, vpWebSocket, vpInbox, $state) {

    vpSystemMessage.get();
    $scope.loginCredentials = {};

    $scope.loginUser = function() {
        var login_promise = vpAccount.login($scope.loginCredentials);
        login_promise.then(function(success) {
            var headers = success.headers();
            vpSession.set(headers.voicepersonalstoken);
            vpWebSocket.setAuthToken(headers.voicepersonalswebsockettoken);
            vpInbox.setNewMessagesCount();
            $state.go('tab.profile-enter-redirect');
        });
    };

    $scope.createNewAccount = function() {
        $state.go('tab.create-account');
    };

})

.controller('CreateAccountCtrl', function($scope, vpAccountCreator, vpSystemMessage, vpConfig) {

    vpSystemMessage.get();
    $scope.newAccount = {};
    $scope.tosVerified = {};

    $scope.showPage = function(page, e) {
        window.open(vpConfig.baseAPIURL() + page, '_system');  //launch system browser
    };

    $scope.createAccount = function() {
        if ($scope.tosVerified.checked) {
            var create_account_promise = vpAccountCreator.get($scope.newAccount);
            create_account_promise.then(function(success) {
                $scope.showSystemMessage("Success", "Your account was successfully created.  Please login to continue.", 'success', 'tab.login');
            });
        } else {
            $scope.showSystemMessage("Error: Terms of Service", "You must read and agree to the terms of service in order to create an account and use this app.", 'error');
        }

    };
})

//doing these redirects outside of personals controller to eliminate flickering of personals template while redirects occur
.controller('ProfileEnterRedirectCtrl', function($scope, $state, vpLoading, vpProfile, vpWebSocket, vpPoller, vpPersonalFilters) {

    vpProfile.get().then(function(profile) {
        if (profile == 'no profile') {
            $state.go('tab.profile-edit');
        } else {
            vpPersonalFilters.initFilters();
            // if (vpWebSocket.browserHasWebSocketSupport()) vpWebSocket.connect();
            // else
                vpPoller.startPolling();
            $state.go('tab.profile');
        }
    });

})


.controller('VerifyAgeCtrl', function($scope, vpConfig, vpAgeVerification, vpSplashScreen) {

    $scope.ageVerificationLoaded = function() {
        vpSplashScreen.hideSplashScreen(true);
    };

    $scope.showPage = function(page) {
        window.open(vpConfig.baseAPIURL() + page, '_system');  //launch system browser
    };

    $scope.confirmedClicked = function() {
        vpAgeVerification.confirmAgeVerified();
        vpAgeVerification.verifyAge();
    };

    $scope.closeApp = function() {
        navigator.app.exitApp();
    };
})

.controller('setupPINSecurityCtrl', function($scope, $state, vpAgeVerification, vpSplashScreen) {

    $scope.pinInput = {};
    $scope.inputErrorMessage = "";

    $scope.pinSetupUILoaded = function() {
        vpSplashScreen.hideSplashScreen(true);
    };

    var decorateErrorMessage = function(message) {
        return '<div class="button button-outline button-block button-positive">'+message+'</div>';
    };

    $scope.setPIN = function( ) {
        if ($scope.pinInput.pin !== undefined) {
            var accessPINToValidate = $scope.pinInput.pin;
            if (accessPINToValidate.length >= 4 && accessPINToValidate.length < 11) {
                if (!isNaN(parseInt(accessPINToValidate)) && isFinite(accessPINToValidate)) {
                    $scope.inputErrorMessage = "";
                    vpAgeVerification.enablePINVerification(accessPINToValidate);
                    $state.go('tab.browse.personals');
                } else $scope.inputErrorMessage = decorateErrorMessage('PIN can only contain only numeric values.');
            } else $scope.inputErrorMessage = decorateErrorMessage("PIN must be 4 to 10 numeric digits.");
        } else $scope.inputErrorMessage = decorateErrorMessage('Please enter a pin first.');
    };

    $scope.optOutPINSecurity = function() {
        vpAgeVerification.optOutPINSecurity();
        $state.go('tab.browse.personals');
    };
})


.controller('VerifyPINCtrl', function($scope, vpAgeVerification, $state, vpSplashScreen) {

    $scope.pinInput = {};

    $scope.verifyPINUILoaded = function() {
        vpSplashScreen.hideSplashScreen(true);
    };

    var decorateErrorMessage = function(message) {
        return '<div class="button button-outline button-block button-assertive">'+message+'</div>';
    };

    $scope.validatePIN = function() {
        var valid = vpAgeVerification.validatePIN($scope.pinInput.pin);
        if (valid) $state.go('tab.browse.personals');
        else $scope.inputErrorMessage = decorateErrorMessage("Sorry, PIN '"+$scope.pinInput.pin+"' is not correct, please try again.");
    };
});

