angular.module('voicePersonals.controllers')

.controller('BrowsePersonalsCtrl', function($scope, vpPersonals, $ionicSlideBoxDelegate, $state, vpProfile,
                                            vpLoading, $ionicPopover, $ionicModal, vpLocationFilters, $q, vpSystemMessage, vpAudioUploader, vpMessenger,
                                            vpSession, vpReplyHandler, vpSystemConfirmAction, vpPersonalFilters, vpSplashScreen) {

    $scope.filterData = vpPersonalFilters.getCurrentFilters();
    $scope.filterLabels = vpPersonalFilters.getFilterLabels();

    vpSystemMessage.get();

    //get personals
    $scope.updatePersonals = function(offset) {
        $scope.baseOffset = $scope.baseOffset + offset;
        var personalsPromise = vpPersonals.get($scope.baseOffset, vpPersonalFilters.getCurrentFilters());
        personalsPromise.then(function(success) {
            vpSplashScreen.hideSplashScreen(true);
            if (success.data.length > 0) {
                if (success.data.length == 1) {
                    $scope.showSystemMessage("No more.", "No more personals are available for filter settings.", 'error');
                    $scope.baseOffset = $scope.baseOffset - offset;
                }
                else {
                    //replace default images with local app versions
                    for (i =0; i < success.data.length; i++) {
	                    if (success.data[i].image_link.indexOf('default_male') > 0)
	                        success.data[i].gender_icon = 'fa-mars';
	                    else if (success.data[i].image_link.indexOf('default_couple') > 0)
	                        success.data[i].gender_icon = 'fa-venus-mars';
	                    else
	                        success.data[i].gender_icon = 'fa-venus';
                    }

                    $scope.personals = success.data;
                    $ionicSlideBoxDelegate.update();
                    $ionicSlideBoxDelegate.slide(0);
                    $scope.setAudioPlayerModel(0);
                    $scope.personalsResponded = [];
                }
            } else $scope.showSystemMessage("None Found", "Sorry, no personals matching your criteria are available.", 'error');

        }, function(error) {
            vpSplashScreen.hideSplashScreen(true);
        });
        return personalsPromise;
    };

    $scope.baseOffset = 0;
    $scope.updatePersonals($scope.baseOffset);

    $scope.personalInFocus;
    $scope.vpMediaPlayerBind = {};
    $scope.setAudioPlayerModel = function(index) {
        $scope.personalInFocus = $scope.personals[index];
        $scope.vpMediaPlayerBind = { url:  $scope.personalInFocus.recording_url };
    };

    $scope.checkPersonalRefresh = function(index) {
        var updateNeeded = (index == ($scope.personals.length - 1)) ? true : false;
        if (updateNeeded) $scope.updatePersonals(index);
        else $scope.setAudioPlayerModel(index);
    };

    //reply options popover menu
    $scope.replyToType = 'Personal';
    $ionicPopover.fromTemplateUrl('templates/reply_options_personal.html', {
        scope: $scope
    }).then(function(popover) {
        $scope.popover = popover;
    });

    $scope.openPopover = function($event) {
        $scope.popover.show($event);
    };
    $scope.closePopover = function() {
        $scope.popover.hide();
    };
    //Cleanup the popover when we're done with it!
    $scope.$on('$destroy', function() {
        if (typeof $scope.popover !== 'undefined') $scope.popover.remove();
    });
    // Execute action on hide popover
    $scope.$on('popover.hidden', function() {
        // Execute action
    });
    // Execute action on remove popover
    $scope.$on('popover.removed', function() {
        // Execute action
    });

    //block user menu choice
    $scope.blockUser = function() {

        if (vpSession.isLoggedIn()) {
            var userConfirmedPromise = vpSystemConfirmAction.showSystemConfirmation('Block User', 'Are you sure you want to block this user?');

            userConfirmedPromise.then(function(success) {

                var block_promise = vpMessenger.blockUser($scope.personalInFocus.account_id);
                block_promise.then(function (success) {
                    $scope.updatePersonals(0);
                });

            }, function(userCancelled) {
                //console.log("user cancelled.");
                ;
            });
        }
        else {
            $scope.showSystemMessage('Login Required', 'Please login to block this user.', 'blocked', 'tab.login', 'Go to Login');
        }

    }


    //reply to personals modal
    $scope.maxRecordingLength = 30;
    $scope.personalsResponded = [];
    $scope.replyMessage = {};   //object binding for reply input form.
    $scope.replyRecordingCompleted = function(filename) {
        $scope.replyMessage.respondingToPersonalID = $scope.personalInFocus.personal_id;
        vpReplyHandler.handleReply($scope, filename, $scope.replyMessage).then(function(success) {
            $scope.personalsResponded.push($scope.personalInFocus.personal_id);
            $scope.showSystemMessage('Message Sent', 'Message Sent Successfully.', 'success');
        });
    }


    $ionicModal.fromTemplateUrl('templates/reply.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function(modal) {
        $scope.modal = modal;
    });
    $scope.replyToPersonal = function() {
        if (vpSession.isLoggedIn()) {
            if (vpProfile.userHasProfile()) {
                if ($scope.personalsResponded.indexOf($scope.personalInFocus.personal_id) > -1) {
                    $scope.showSystemMessage('Request Cancelled', "You have already responded to this personal.", 'blocked');
                    return;
                }
                $scope.modal.show();
            } else {
                $scope.showSystemMessage('Profile Required', 'Please create your profile to reply to personals.', 'blocked', 'tab.profile-enter-redirect', 'Go to Profile');
            }
        }
        else {
            $scope.showSystemMessage('Login Required', 'Please login to reply to personals.', 'blocked', 'tab.login', 'Go to Login');
        }
    };
    $scope.closeModal = function() {
        $scope.modal.hide();
    };
    //Cleanup the modal when we're done with it!
    $scope.$on('$destroy', function() {
        if (typeof $scope.modal !== 'undefined') $scope.modal.remove();
    });
    // Execute action on hide modal
    $scope.$on('modal.hidden', function() {
        // Execute action
    });
    // Execute action on remove modal
    $scope.$on('modal.removed', function() {
        // Execute action
    });


    //filter personals modal
    //object to construct the inputs, these should probably be stored in the config, can change depending on application
    $scope.authorGenderOptions = vpPersonalFilters.getAuthorGenders();
    $scope.seekingGenderOptions = angular.copy($scope.authorGenderOptions);
    $scope.typeOptions = vpPersonalFilters.getRelationshipTypeOptions();
    $scope.locationOptions = [];
    $ionicModal.fromTemplateUrl('templates/filter_personals.html', {
        scope: $scope
        ,animation: 'slide-in-up'
    }).then(function(modal) {
        $scope.filterModal = modal;
    });
    $scope.showFilterPersonals = function(filter_type) {
        $scope.filter_type = filter_type;
        if ($scope.filter_type == 'Location') {
            var locationFilterPromise = vpLocationFilters.get();
            locationFilterPromise.then(function(success) {
                $scope.locationOptions = [{label: 'All', value: 'all'}];
                for (var key in success.data) {
                    if (success.data.hasOwnProperty(key)) {
                        $scope.locationOptions.push({label: success.data[key], value: key});
                    }
                }
                $scope.filterModal.show();
            });
        } else $scope.filterModal.show();
    };
    $scope.hideFilterPersonals = function() {
        $scope.filterModal.hide();
    };
    //Cleanup the modal when we're done with it!
    $scope.$on('$destroy', function() {
        if (typeof $scope.filterModal !== 'undefined') $scope.filterModal.remove();
    });
    // Execute action on hide modal
    $scope.$on('modal.hidden', function() {
        // Execute action
    });
    // Execute action on remove modal
    $scope.$on('modal.removed', function() {
        // Execute action
    });

    //personals filtering form
    $scope.filterPersonals = function(label) {
        $scope.baseOffset = 0;
        $scope.updatePersonals($scope.baseOffset).then(function() {
            for (var filterType in label) {
                if (label[filterType] == 'Show All') label[filterType] = 'All';
                vpPersonalFilters.setFilterLabel(filterType, label[filterType]);
                $scope.hideFilterPersonals();
            }
        });
    }

});