angular.module('voicePersonals.controllers')

.controller('ProfileEditCtrl', function($scope, vpSession, $state, $ionicSlideBoxDelegate, $sce, $timeout, vpGeolocation, vpSystemMessage, vpDatePicker, vpAudioUploader, vpProfile, vpLoading, $state, vpConfig) {

    if (!vpSession.isLoggedIn()) $state.go('tab.login');  //should never happen but force logged in state JIC

    vpSystemMessage.get();

    $scope.personal = {};
    $scope.maxRecordingLength = 30;
    $scope.category = vpConfig.getCategories();

    var Permission = {};
    if (window.hasOwnProperty('cordova'))
        Permission = window.plugins.Permission;  ///TODO re-enable, needed for cordova

    var requiredPermissions = ['android.permission.RECORD_AUDIO', 'android.permission.WRITE_EXTERNAL_STORAGE'];
    $scope.audio_permission_status = $sce.trustAsHtml("<span style='color: white; background-color: black;'>Pending</span>");
    $scope.storage_permission_status = $sce.trustAsHtml("<span style='color: white; background-color: black;'>Pending</span>");

    $scope.startWorkflow = function(index) {
        $ionicSlideBoxDelegate.enableSlide(false);
        if (window.hasOwnProperty('cordova')) $scope.verifyPermissions();
        else $scope.handleChangeAsync();
    };

    this.currentSlideIndex = 0;
    $scope.handleChange = () => {
        this.currentSlideIndex++;
        $ionicSlideBoxDelegate.slide(this.currentSlideIndex, 500);
    };

    $scope.handleChangeAsync = function() { //use this for testing, disable verifyPermission
        try {
            $scope.handleChange();
        } catch (error) {
            $scope.handleFailedChange();
            $timeout($scope.handleChangeAsync, 1000);
        }
    };

    $scope.handleFailedChange = () => {
        this.currentSlideIndex--;
    }

    /*
    $scope.setLocation = function() {
        var loc_promise = vpGeolocation.getLocation();
        loc_promise.then(function(data) {
            console.log("Geolocation received in profile edit: ", data);
        }, function(error) {
            $scope.showSystemMessage("Error", error, 'error');
        });
    }; */

    $scope.saveTitle = function() {
        if (!$scope.personal.hasOwnProperty('title'))
            $scope.personal.title = 'Voice Personal';
        $scope.handleChange();
    }

    $scope.showDatePicker = function() {
        if (window.hasOwnProperty('cordova')) {
            vpDatePicker.showDatePicker().then(function(date) {
                $scope.personal.birthDate = date;
                $scope.handleChange();
            }, function(error) {
                if (error != 'Cancelled') $scope.showSystemMessage(error);
            });
        } else {
            $scope.personal.birthDate = '1982-05-01';
            $scope.handleChange();
        }

    };

    $scope.recordingCompleted = function(filename) {  //The directive has completed an audio recording
        $scope.personal.localAudioFileName = filename;
        //$scope.handleChange();  //re-enable on picture implementation
        $scope.uploadPersonal();
    };

    $scope.addPicture = function() {
        $scope.uploadPersonal();
    };

    $scope.uploadPersonal = function() {

        vpLoading.showLoading();

        //upload the audio file.
        var promise = vpAudioUploader.uploadAudio($scope.personal.localAudioFileName);
        promise.then(function(recordingURL) {

            vpLoading.hideLoading();

            $scope.personal.recordingURL = recordingURL;

            promise = vpProfile.upload($scope.personal);
            promise.then(function(success) {
                $scope.showSystemMessage('Personal Created', 'Your personal was successfully created.', 'success', 'tab.profile-enter-redirect');
            });

        }, function(error) {
            vpLoading.hideLoading();
            // console.log("error uploading audio file: ", error);
            $scope.showSystemMessage('Audio Upload Error', 'Sorry.  There was an error uploading your audio file.<br /><br />Please try again later.', 'error');
        });

        //upload the personal image

    };


    //verify permissions
    $scope.verifyPermissions = function() {
        Permission.has(requiredPermissions, function(results) {
            var neededPermissions = [];
            for (var current_permission in results) {
                if (results.hasOwnProperty(current_permission)) {
                    if (!results[current_permission]) {
                        neededPermissions.push(current_permission);
                        if (current_permission == 'android.permission.RECORD_AUDIO')
                            $scope.audio_permission_status = $sce.trustAsHtml("<span style='color: yellow; background-color: black;'>Missing</span>");
                        else if (current_permission == 'android.permission.WRITE_EXTERNAL_STORAGE')
                            $scope.storage_permission_status = $sce.trustAsHtml("<span style='color: yellow; background-color: black;'>Missing</span>");
                    } else {
                        if (current_permission == 'android.permission.RECORD_AUDIO')
                            $scope.audio_permission_status = $sce.trustAsHtml("<span style='color: green;'>OK</span>");
                        else if (current_permission == 'android.permission.WRITE_EXTERNAL_STORAGE')
                            $scope.storage_permission_status = $sce.trustAsHtml("<span style='color: green;'>OK</span>");
                    }
                }
            }

            if (neededPermissions.length > 0) {
                Permission.request(neededPermissions, function(grants) {
                    console.log('Grants after asking for permissions: ', grants);
                }, function(error) {
                    console.log("Error requesting permission, error: ", error);
                });
            } else {  //slider may not be initialized, hence the async try catch function stuff
                $scope.handleChangeAsync();
            }

        }, function(error) {
            console.log("Error verifying RECORD_AUDIO permission, error: ", error);
        });
    }

});