angular.module('voicePersonals.directives')

.directive('vpLoveAudioRecorder', function(vpAudioRecorder, vpSystemMessage, $rootScope, $interval) {

        vpSystemMessage.get();

        return {
            templateUrl: 'templates/vp-love-audio-recorder.html',
            restrict: 'E',

            scope: {  //isolate scope
                recordingCompleted: '=',
                maxLengthSecs: '='
            },

            link: function(scope, element, attr) {

                scope.initRecorder = function() {
                    scope.audioRecording = false;
                    scope.recordingSaved = false;
                    scope.recordingLengthInterval = null;
                    scope.currentRecordingLength = null;
                    scope.recordingToVerify = null;
                    scope.cancelingRecording = false;
                };

                scope.initRecorder();

                scope.recordAudio = function() {

                    scope.currentRecordingLength = 0;
                    scope.audioRecording = true;
                    scope.recordingLengthInterval = $interval(scope.updateRecordingLength, 1000);

                    var promise = vpAudioRecorder.recordAudio(scope.maxLengthSecs);
                    promise.then(function(fileName) {
                        scope.recordingToVerify = {url: fileName};
                        scope.recordingSaved = true;
                        scope.audioRecording = false;
                        $interval.cancel(scope.recordingLengthInterval);
                        if (scope.cancelingRecording) //user requested to cancel any on-going recording, skip verification and reset
                            scope.restartRecording();

                    }, function(error) {
                        console.log("Error recording audio: ", error);
                        $rootScope.showSystemMessage("Error Recording Audio", 'Sorry.  There was an error recording your audio.<br /><br />Please try again later.', 'error');
                    });
                };

                scope.finishRecording = function() {
                    $interval.cancel(scope.recordingLengthInterval);
                    vpAudioRecorder.finishAudioRecording();  //this should cause the promise to invoke its success state defined in recordAudio
                };

                scope.recordingVerified = function() {
                    scope.recordingCompleted(scope.recordingToVerify.url);
                    scope.initRecorder();  //reset for next recording
                };

                scope.restartRecording = function() {
                    scope.initRecorder();
                };

                scope.updateRecordingLength = function() {
                    scope.currentRecordingLength++;
                };

                scope.getRecordingStatus = function() {
                    if (scope.audioRecording) return 'recording';
                    else if (scope.recordingSaved) return 'verification';
                    else return 'init';
                };

                scope.cancelingRecording = false;
                scope.$on('replyToPersonalCanceled', function(event, data) {  //user cancelled recording, stop any recording happening and reset UI
                    scope.cancelingRecording = true;
                    if (scope.audioRecording) scope.finishRecording();
                    else scope.restartRecording();  //finishRecording will invoke this implicitly
                });

                element.on('$destroy', function() {  // need this once implement maximum recording length?
                    $interval.cancel(scope.recordingLengthInterval);
                });
            }
        }
});