angular.module('voicePersonals.directives')

.directive('vpLoveMediaPlayer', function($rootScope, vpLoading, vpFileTransfer, vpMedia, $interval, vpSystemMessage, vpHTTP) {

    return {

        templateUrl: 'templates/vp-love-media-player.html',
        restrict: 'E',

        scope: {  //isolate scope
            audioSource: '=audio',
            localFile: '='
        },

        link: function(scope, element, attr) {

            scope.mediaPlayer = null;
            scope.audioPlaying = false;
            scope.audioPaused = false;  //can only be paused once playing
            scope.audioLength = 0;
            scope.audioPosition = 0;

            scope.playPositionInterval = null;
            scope.getDurationInterval = null;

            scope.playAudio = function(audioURL) {

                vpLoading.showLoading(scope);

                if (scope.localFile) {
                    scope.audioPlaying = true;
                    scope.mediaPlayer = vpMedia.newMedia(audioURL, scope.mediaSuccess(), scope.mediaError(), scope.mediaStatus);
                    vpMedia.play(scope.mediaPlayer);
                }
                else {  //loading remote file
                    if (audioURL.indexOf('api/personal/getPersonalRecordingURL') != -1) {  //translate url to recording
                        vpHTTP.get(audioURL, 'get recording location', true, true).then(function(uriObj) {
                            downloadPlay(JSON.parse(uriObj.data));
                        }, function error(err) {
                            // console.log("error get uri mapped", err);
                        });
                    } else downloadPlay(audioURL);
                }
            };

            function downloadPlay(uri) {
                vpFileTransfer.downloadFile(uri, 'downloads/current_audio.mp3', false) //overwrites each time.
                    .then(function(download) {  //vpLoading is closed by receiving mediaStatus media running
                        vpLoading.hideLoading();
                        scope.audioPlaying = true;
                        download_path = download.nativeURL;
                        scope.mediaPlayer = vpMedia.newMedia(download_path, scope.mediaSuccess, scope.mediaError, scope.mediaStatus);
                        vpMedia.play(scope.mediaPlayer);
                    }, function(error) {
                        vpLoading.hideLoading();
                        $rootScope.showSystemMessage('Playback Error', 'There was an error trying to play the audio.<br /><br />Please try again later.', 'error');
                    }, function(status) {
                        // console.log("download file status received: "+JSON.stringify(status));
                    });
            }

            scope.isAudioPlaying = function() {
                return scope.audioPlaying;
            };

            scope.mediaSuccess = function() {
                // console.log("media success");
            };
            scope.mediaError = function(error) {
                // console.log("media playback error: ", error);
            };
            scope.mediaStatus = function(status) {

                switch(status) {
                    case Media.MEDIA_RUNNING:
                        vpLoading.hideLoading();
                        scope.playPositionInterval = $interval(scope.updateAudioPlayPosition, 1000);
                        scope.getDurationInterval = $interval(scope.updateDuration, 1000);
                        break;
                    case Media.MEDIA_PAUSED:
                        $interval.cancel(scope.playPositionInterval);
                        break;
                    case Media.MEDIA_STOPPED:  //stopped
                        scope.audioPaused = true;
                        $interval.cancel(scope.playPositionInterval);
                        break;
                    default:
                    //alert("received status: "+status);

                }
            };

            scope.updateAudioPlayPosition = function() {
                vpMedia.getCurrentPosition(scope.mediaPlayer).then(function(result) {
                    if (result > 0) {  //negative value possible after finished playing
                        scope.audioPosition = result;
                    }
                });
            };

            scope.updateDuration = function() {
                vpMedia.getDuration(scope.mediaPlayer).then(function(seconds) {
                    scope.audioLength = Math.floor(seconds);
                    $interval.cancel(scope.getDurationInterval);
                });
            };

            scope.pauseResumeAudio = function() {
                if (scope.audioPaused) {
                    scope.audioPaused = false;
                    vpMedia.play(scope.mediaPlayer);
                }
                else {
                    scope.audioPaused = true;
                    vpMedia.pause(scope.mediaPlayer);
                }
            };
            //add an on destroy function to cancel all interval handlers that exist

            scope.$watch("audioSource.url", function(newValue, oldValue) {
                if (scope.mediaPlayer != null) {
                    scope.mediaPlayer.release();
                    scope.mediaPlayer = null;
                }
                //reset media player parameters
                scope.audioPlaying = false;
                scope.audioPaused = false;
                scope.audioLength = 0;
                scope.audioPosition = 0;
                $interval.cancel(scope.getDurationInterval);
            });

            element.on('$destroy', function() {
                $interval.cancel(scope.playPositionInterval);
                $interval.cancel(scope.getDurationInterval);
            });
        }

    }
});

