var voice_personals_module = angular.module('voicePersonals.services', []);

voice_personals_module.factory('vpLoading', function($ionicLoading) {

    return {
        showLoading : function() {  //send in scope otherwise artifacts can show
            $ionicLoading.show({
                template: '<i class="icon ion-loading-b"></i><br />Loading...'
            });
        },

        hideLoading : function(){
            $ionicLoading.hide();
        }
    }

});

voice_personals_module.factory('vpPersonals', function(vpConfig, vpHTTP) {
    return {
        get: function (offset, filters) {

            var url = vpConfig.baseAPIURL() + "/personal/personalsJSON";

            //filters
            var gender_url = 'none';
            var type_url = 'none';
            var location_url = 'none';
            if (typeof filters !== 'undefined') {
                var authorGenderDefined = (typeof filters.authorGender !== 'undefined') && (filters.authorGender !== 'all');
                var genderSeekingDefined = (typeof filters.seekingGender !== 'undefined') && (filters.seekingGender !== 'all');
                if (authorGenderDefined || genderSeekingDefined) {
                    var author_gender_url = (authorGenderDefined) ? filters.authorGender : 'allJSONOnly';
                    var seeking_gender_url = (genderSeekingDefined) ? filters.seekingGender : 'allJSONOnly';
                    gender_url = author_gender_url+"-seeking-"+seeking_gender_url;
                }
                if (typeof filters.type !== 'undefined' && filters.type !== 'all') type_url = filters.type;
                if (typeof filters.location !== 'undefined' && filters.location !== 'all') location_url = filters.location;
            }
            url = url+"/"+gender_url+"/"+type_url+"/"+location_url;

            //offset
            offset = typeof offset !== 'undefined' ? offset : 0;
            if (offset) url = url+"/"+offset;

            return vpHTTP.get(url, 'load personals');
        }
    };
});

voice_personals_module.factory('vpLocationFilters', function(vpConfig, vpHTTP) {
    return {
        get: function() {
            var url = vpConfig.baseAPIURL() + '/api/personal/getLocationFilters';
            return vpHTTP.get(url, 'load location filters');
        }
    }
});

voice_personals_module.factory('vpAccountCreator', function(vpConfig, vpHTTP) {
    return {
        get: function(newAccount) {
            newAccount.deviceType = 2;  //android
            var url = vpConfig.baseAPIURL() + '/api/account/createAccount';
            return vpHTTP.post(url, newAccount, 'create your account');
        }
    }
});

voice_personals_module.factory('vpAccount', function(vpConfig, vpHTTP) {
    return {
        login: function(loginCred) {
            var url = vpConfig.baseAPIURL() + '/api/account/login';
            return vpHTTP.post(url, loginCred, 'login');
        },
        logout: function() {
            var url = vpConfig.baseAPIURL() + '/api/account/logoutAPI';
            return vpHTTP.post(url, null, 'logout');
        },
        getNewWebSocketToken: function() {
            var url = vpConfig.baseAPIURL()+'/api/account/getNewWebSocketToken';
            return vpHTTP.post(url, null, 'retrieve web socket authorization');
        }
    }
});

//Uploads a file to the server that was recorded with the audio recorder directive
voice_personals_module.factory('vpAudioUploader', function(vpFileTransfer, vpConfig, $q) {

    return {
        uploadAudio: function(fileName) {
            var q = $q.defer();
            var uploadPromise = vpFileTransfer.uploadFile(fileName, 'audio/amr', vpConfig.baseAPIURL()+'/api/personal/uploadAudioFile');
            uploadPromise.then(function(result) {
                q.resolve(JSON.parse(result.response).recording_url);
            }, function(error) {
                q.reject("Error uploading file: "+error.body);
            });
            return q.promise;
        }
    }

});


voice_personals_module.factory('vpMessenger', function(vpConfig, vpHTTP) {

    return {
        upload: function(replyMessage) {
            if (replyMessage.hasOwnProperty('respondingToPersonalID')) {
                var url = vpConfig.baseAPIURL()+'/api/message/sendMessage';
                return vpHTTP.post(url, replyMessage, 'send your message');
            }
            else if (replyMessage.hasOwnProperty('respondingToMessageID')) {
                var url = vpConfig.baseAPIURL()+'/api/message/reply';
                return vpHTTP.post(url, replyMessage, 'send your message');
            }
        },
        deleteMessage: function(messageID) {
            var url = vpConfig.baseAPIURL()+'/api/message/delete';
            return vpHTTP.post(url, {message_id: messageID}, 'delete your message');
        },
        blockUser: function(userID) {
            var url = vpConfig.baseAPIURL()+'/api/account/blockUser';
            return vpHTTP.post(url, {user_id_to_block: userID }, 'block the user');
        }
    }

});

