var voice_personals_services_module = angular.module('voicePersonals.services');

voice_personals_services_module.factory('vpAgeVerification', function($state, $timeout, $rootScope, $q,
  vpFileTransfer, vpSystemMessage, vpLoading, vpLocalStorage, vpInbox, vpWebSocket, vpSession, vpConfig) {

    vpSystemMessage.get();

    var stateOnResume = 'tab.browse.personals';

    var isAgeVerified = function() {
        return (vpLocalStorage.get('ageVerified') == 'true');
    }

    var pinAccessOptioned = function() {   //whether or not the user has decided to use PIN verification
        return (vpLocalStorage.get('pinAccessOptioned') == 'true');
    }

    var setPinAccessOptioned = function() {
        vpLocalStorage.set('pinAccessOptioned', 'true');
    }

    var isPINRequired = function() {
        return (vpLocalStorage.get('isPINRequired') == 'true');
    }

    var setPINIsRequired = function(value) {
        vpLocalStorage.set('isPINRequired', value);
    }

    var getPIN = function() {
        return (vpLocalStorage.get('vpPINToVerifyOnAppResume'));
    }

    var setPIN = function(pin) {
        vpLocalStorage.set('vpPINToVerifyOnAppResume', pin);
    }

    return {

        /*
         *  System starts here.
         *  Checks age verification on first load.
         *  If user has PIN security enabled, prompt.
         *
         *  Sessions are assumed at this point to have very long expirations.
         */
        verifyAge: function() {

            //this gets called on app resume activate web socket, set login headers, and check for new messages
            vpInbox.setNewMessagesCount(true, true);  //do this first, special error messaging for unreachable host.
            vpSession.refreshLogin();
            // vpWebSocket.connect();

            if (vpConfig.getUbbseenTooToo57Warning())
            {
                if (isAgeVerified()) {
                    if ( ! pinAccessOptioned()) { $state.go('setupPINSecurity'); }
                    else if (isPINRequired()) { $state.go('verifypin'); }
                    else { $state.go(stateOnResume); } //user has opted out of PIN security
                }
                else { $state.go('verifyage'); }
            } else $state.go(stateOnResume);
        },

        pauseCordova: function() {
            if (isPINRequired()) $state.go('verifypin');
            // vpWebSocket.close();
            $rootScope.closeSystemMessage();
        },

        confirmAgeVerified: function() {
            vpLocalStorage.set('ageVerified', 'true');
        },

        enablePINVerification: function(pinInput) {
            setPIN(pinInput);
            setPINIsRequired('true');
            setPinAccessOptioned();
        },

        optOutPINSecurity: function() {
            setPINIsRequired('false');
            setPinAccessOptioned();
        },

        validatePIN: function(pin_input) {
            var valid = (getPIN() == pin_input) ? true : false;
            return valid;
        }
    }
});
