var services_module = angular.module('voicePersonals.services');


services_module.factory('vpFileTransfer', function($q, $window) {

    return {

        /*********************************************************************
         *
         * File Upload and Download functions from remote URIs.
         *
         *********************************************************************/


        downloadFile: function (source, path, trustAllHosts, options) {
            var q = $q.defer();

            getFilesystem().then(function(fileSystem) {
                var file_url = fileSystem.root.toURL()+path;
                var fileTransfer = new FileTransfer();
                var uri = encodeURI(source);
                fileTransfer.onprogress = q.notify;
                fileTransfer.download(uri, file_url, q.resolve, q.reject, false, options);
            }, function(error) {
                q.reject(error);
            });

            return q.promise;
        },

        uploadFile: function (fileNameToUpload, fileMIME, uploadURL) {
            var q = $q.defer();

            getFilesystem().then(function(fileSystem) {


                //var file_to_upload = fileSystem.root.toURL()+fileNameToUpload;
                //media plugin saves recorded audio to external app storage
                var path_to_file = cordova.file.externalRootDirectory+fileNameToUpload;

                var fileTransfer = new FileTransfer();
                fileTransfer.onprogress = q.notify;

                var options = new FileUploadOptions();
                options.fileKey = "file";
                options.fileName = fileNameToUpload;
                options.mimeType = fileMIME;
                options.chunkedMode = false;  //required cause apache rejects 0 content-length post requests

                uploadURL = encodeURI(uploadURL);

                fileTransfer.upload(path_to_file, uploadURL, q.resolve, q.reject, options);

            }, function(error) {
                q.reject(error);
            });

            return q.promise;
        },


        /*************************************************************
         *
         *  File IO functions.
         *
         *  Note: these files persist between browsers sessions and restarts,
         *  cordova application stops and starts,
         *  and cordova application re-installs, application upgrades,
         *  and manual unistallations.
         *
         *  Note that this is not supported in ios.  Or else the ios does not support
         *  indefinite storage, although I've read about another mechanism...chain something
         *  or another?
         *
         ***************************************************************/

        //this is an example of a way to save a file with error handling.
        //this should be incorporated into this class and made the public interface,
        //hiding the lower level vpFileTransfer.saveToFile function.

        //if (saveCount == 0) savePromise = $q.defer();
        //
        //vpFileTransfer.saveToFile(pin, savedPINFilename).then(function() {  //how to test this looping?
        //    savePromise.resolve("pin saved");
        //}, function(fail) {
        //    saveCount++;
        //    console.log("failed to save access pin to disk: ");
        //    console.log(fail);
        //    if (saveCount < 6) {
        //        console.log("re-attempting to save on attempt count: "+saveCount);
        //        $timeout(function() { savePINToDisk(saveCount); }, 1000);
        //    }
        //    else {
        //        savePromise.reject("failed to save pin");
        //    }
        //
        //});
        //
        //if (saveCount == 0) return savePromise.promise;

        saveToFile: function (stringToSave, fileName) {
            var q = $q.defer();

            getFilesystem().then(function(fileSystem) {
                 fileSystem.root.getFile(fileName, {create: true, exclusive: false}, function(fileEntry) {
                     fileEntry.createWriter(function(writer) {
                         writer.write(stringToSave);
                         q.resolve("pin saved");
                     } , function() { q.reject("failed to createWriter");});
                 }, function() { q.reject("failed to getFile");});
            }, function(error) { q.reject(error);});

            return q.promise;
        },



        //this is an example of a way to read a file with error handling.
        //this should be incorporated into this class and made the public interface,
        //hiding the lower level vpFileTransfer.readFile function.

        //if (readCount == 0) readPromise = $q.defer();
        //
        //vpFileTransfer.readFile(savedPINFilename, false).then(function(result) {
        //    readPromise.resolve(result);
        //}, function(fail) {
        //    readCount++;
        //    if (readCount < 6) {
        //        $timeout(function() { readPINFromDisk(readCount); }, 1000);
        //    }
        //    else {
        //        console.log("readFile failed, error: ");
        //        console.log(fail);
        //        readPromise.reject('failed to read pin from disk');
        //    }
        //});
        //
        //if (readCount == 0) return readPromise.promise;



        //fileRequired - set to true to set error state if file does not exist, false if it is expected and allowed file can be missing.
        readFile: function(fileName, fileRequired) {
            var q = $q.defer();

            getFilesystem().then(function(fileSystem) {
                fileSystem.root.getFile(fileName, null, function(fileEntry) {
                    fileEntry.file(function(file) {
                            var reader = new FileReader();
                            reader.onloadend = function (evt) {
                                q.resolve(evt.target.result);
                            };
                            reader.readAsText(file);
                        } , function() {q.reject("failed to get file to read");});
                }, function(error) {
                    if (error.code == 1 && !fileRequired) q.resolve("File does not exist.");  //file does not exist error, is allowed?
                    else q.reject("failed to get file entry for read");
                });
            }, function(error) { q.reject(error);});

            return q.promise;
        },

        deleteFile: function(fileName) {
            var q = $q.defer();

            getFilesystem().then(function(fileSystem) {
                fileSystem.root.getFile(fileName, null, function(fileEntry) {
                    fileEntry.remove(function(success) {
                        q.resolve("File deleted.");
                    }, function(error) {
                        q.reject("Error Deleting File: ");
                    });
                }, function() {q.reject("failed to get file entry for read");});
            }, function(error) { q.reject(error);});

            return q.promise;
        }
    }

    function getFilesystem() {
        var q = $q.defer();
        try {
            $window.requestFileSystem($window.PERSISTENT, 0, q.resolve, q.reject);
        } catch (err) {
            q.reject(err);
        }
        return q.promise;
    }

});
