var voice_personals_services_module = angular.module('voicePersonals.services');

voice_personals_services_module.factory('vpInbox', function(vpConfig, $q, vpSession, vpHTTP, $rootScope, vpConfig) {

    var vpInboxService = this;

    this.currentInboxView = 'none';
    this.messages = {new: [], sent: [], deleted: []};  //must use arrays otherwise order of ng-repeat goes out
    this.inboxLoaded = false;
    this.sessionToken = null;  // for last resort access verification, JIC.


    return {

        /*
         *  Invoked in 4 cases:
         *  1.  To init new message count after login (no UI block).
         *  2.  To get new message count when blocking users (UI blocked, but why?  Doesn't blocking have its own UI block?).
         *  3.  By poller as fall back when web sockets are not available. (no UI block).
         *  4.  On system resume from age verification controller.  (no UI block).
         *
         *  Note:  This is one of the first things (web socket) that happens when starting the app.  An error will be reported if
         *  the application host is unreachable (IE, blocked by Service Provider Adult Settings while changing networks?),
         *  provide guidance.
         *
         */
        setNewMessagesCount: function(noUIBlock, appStart) {

            var url = vpConfig.baseAPIURL() + '/api/message/getNewMessageCount';

            if (typeof appStart === 'undefined')
                var errorMsg = 'update your messages';
            else {
                var errorMsg = 'update your messages.  ' +
                    '<br /><br/>Please verify if you are able to access <a href="#" onclick="window.open(\''+vpConfig.baseAPIURL()+'\', \'_system\'); ">'+vpConfig.baseAPIURL()+'</a>'+
                    '<br /><br />We may be experiencing an outage or perhaps your network\'s service provider is restricting your access';
            }

            //skip login redirect if login is not found valid
            //Need to run this on resume to verify host access but don't want to affect app resume and age verification flow
            //this means sometimes a user will not be logged in and that is expected
            var skipLoginRedirect = true;

            vpHTTP.get(url, errorMsg , noUIBlock, skipLoginRedirect).then(function(success){
                $rootScope.checkIfNewMsgReceived(success.data);  //turns on blinker if new messages found since last time app used
                $rootScope.initNewMsgCount(success.data);
            });
        },

        getMessages: function (forceUpdate, messageType, errorMsgAction) {

            var deferred = $q.defer();

            if (!vpInboxService.inboxLoaded || forceUpdate) {

                var url = vpConfig.baseAPIURL() + '/api/message/getMessages';
                vpHTTP.get(url, errorMsgAction).then(function (success) {
                    vpInboxService.messages = {new: [], sent: [], deleted: []};
                    for (var messageTypeLoop in success.data) {
                        if (success.data.hasOwnProperty(messageTypeLoop)) {
                            vpInboxService.messages[messageTypeLoop] = [];
                            for (var i = 0; i < success.data[messageTypeLoop].length; i++) {
                                vpInboxService.messages[messageTypeLoop].push(success.data[messageTypeLoop][i]);
                            }
                        }
                    }

                    $rootScope.initNewMsgCount(vpInboxService.messages['new'].length);
                    $rootScope.storeCurrentMsgCount(vpInboxService.messages['new'].length);
                    vpInboxService.inboxLoaded = true;
                    vpInboxService.sessionToken = vpSession.getToken();
                    deferred.resolve(vpInboxService.messages[messageType]);

                }, function (error) {

                    deferred.reject("An error occurred retrieving your messages.  Please try again later.");

                });

            }
            else {
                if (vpSession.getToken() != vpInboxService.sessionToken) this.handleLogout();  //should never happen but JIC
                deferred.resolve(vpInboxService.messages[messageType]);
            }

            return deferred.promise;

        },

        getCurrentInboxView: function() {
            return vpInboxService.currentInboxView;
        },

        setCurrentInboxView: function(current) {
            vpInboxService.currentInboxView = current;
        },

        getCurrentInboxViewDisplayName: function() {
            return vpInboxService.currentInboxView[0].toUpperCase() + vpInboxService.currentInboxView.slice(1);
        },

        handleLogout: function() {
            vpInboxService.messages = {new: [], sent: [], deleted: []};  //must use arrays otherwise order of ng-repeat goes out
            vpInboxService.inboxLoaded = false;
            $rootScope.disableNewMessageAlert(true);
        }

    }

});
