var voice_personals_services_module = angular.module('voicePersonals.services');

voice_personals_services_module.factory('vpWebSocket', function($rootScope, $timeout, $interval, vpConfig, vpSession, vpAccount, vpProfile) {

    const SOCKET_CLOSED_NORMAL = 1000;
    const PING_MINUTES_FREQ = 5;

    var webSocket;
    var authToken;
    var pingInterval;  //KEEP-ALIVE PINGS

    var stopKeepAlivePing = function() {
        if (pingInterval != null) {
            $interval.cancel(pingInterval);
            pingInterval = null;
        }
    }


    var initVars = function() {
        //console.log("initvars invoked");
        webSocket = null;
        //authToken = null;  //only remove once used,  expires anyhow
        stopKeepAlivePing();
    }
    initVars();

    var connectSocket = function() {
        console.log("Web sockets are disabled.  This should not happen.");
        // if (webSocket == null) {
        //     webSocket = new WebSocket(vpConfig.webSocketURI());
        //     webSocket.onmessage = onSocketMessage;
        //     webSocket.onopen = onSocketOpen;
        //     webSocket.onerror = onSocketError;
        //     webSocket.onclose = onSocketClosed;
        // }
    }

    //Sends message to the serve to enact ping pong exchange
    var pingOnInterval = function() {
        if (webSocket !== null) {
            //console.log("pinging web socket keep alive");
            webSocket.send(JSON.stringify({KeepAlivePing: true}));
        } else {
            initVars();
        }
    };

    var gettingNewAuthToken = false;
    var onSocketMessage = function(message) {   //incoming web socket messages
        var data = JSON.parse(message.data);
        if (data.web_socket_connection_valid) {
            //console.log("web socket connection valid");
            authToken = null;  //one time use, unset token, onopen is called even if open fails
            stopKeepAlivePing();  //stop any old one before starting new one.
            pingInterval = $interval(pingOnInterval, PING_MINUTES_FREQ * 60000);  //milliseconds in 1 minute
        }
        else if (data.token_verification_failed) {  //token expired or already used (maybe downed server or something)
            //console.log("token verification failed, getting new one and attempting reconnect.");
            if (!gettingNewAuthToken) {  //close current socket, renew new token, reconnect.
                gettingNewAuthToken = true;
                webSocket.close();
                vpAccount.getNewWebSocketToken().then(function(result) {
                    var headers = result.headers();
                    authToken = headers.voicepersonalswebsockettoken;
                    gettingNewAuthToken = false;
                    connectSocket();
                }, function(error) {
                    //console.log("getting web socket token failed, no login?");
                    gettingNewAuthToken = false;
                    //initVars();  //don't need this, webSocket.close() above will initVars.
                });
            }
        }
        else if (data.new_message) $rootScope.setNewMessageReceived();
        //else if (data.pong)  console.log("Pong from a keep alive ping.");  //note: to test this, pongs must be implemented in web socket server

    }

    var onSocketOpen = function(e) {  //on connection, send Authorization Token to be used as Client Identifier
        //console.log("on socket open invoked.");
        vpProfile.get(true).then(function(profile) {
            webSocket.send(JSON.stringify({VoicePersonalsAuthorization: authToken, accountID: profile.account_id}));
        }, function(error) {  //socket connection failed, user probably not logged in.
            initVars();  //reset socket for next login.
        });

    }

    var onSocketClosed = function(e) {
        //console.log("on socket closed invoked");
        initVars();
        if (e.code != SOCKET_CLOSED_NORMAL) {  //socket is closed, but is expected to be open.  Retry connection.
            //console.log("\nabnormal socket closed detected:");
            //console.log(e);
            $timeout(connectSocket, 60000);
        }
    }

    var onSocketError = function(e) {  //error too generic to do anything except log.
        // console.log("\non socket error detected:");
        // console.log(e);
    }

    return {

        connect: function() {  //first called on login, but then invoked by self to reconnect if connection fails or some other error
            connectSocket();
        },

        close: function() {
            if (webSocket != null) webSocket.close();  //on socket close cleans up variables
        },

        setAuthToken: function(auth) {
            authToken = auth;
        },

        browserHasWebSocketSupport: function() {
            if ('WebSocket' in window && (typeof WebSocket === 'function' || typeof WebSocket === 'object'))
                return true;
            else
                return false
        }

    }

});